@@ -21,8 +21,8 @@
 #' if TRUE only the flowline network and attributes will be returned
 #' @param streamorder integer only streams of order greater than or equal will be downloaded.
 #' Not implemented for local data.
-#' @details If \code{\link{stage_national_data}} has been run in the current
-#' session, this function will use the staged national data automatically.
+#' @param out_prj character override the default output CRS of NAD83 lat/lon (EPSG:4269)
+#' @details
 #'
 #' This function relies on the National Seamless Geodatabase or Geopackage.
 #' It can be downloaded
@@ -116,7 +116,7 @@
 
 subset_nhdplus <- function(comids = NULL, output_file = NULL, nhdplus_data = NULL, bbox = NULL,
                            simplified = TRUE, overwrite = FALSE, return_data = TRUE, status = TRUE,
-                           flowline_only = NULL, streamorder = NULL) {
+                           flowline_only = NULL, streamorder = NULL, out_prj = 4269) {
 
   if(is.null(flowline_only)) {
     if(!is.null(nhdplus_data) && nhdplus_data == "download") {
@@ -126,6 +126,8 @@ subset_nhdplus <- function(comids = NULL, output_file = NULL, nhdplus_data = NUL
     }
   }
 
+  if(!is.null(comids) && length(comids) == 0) stop("comids must be NULL or non-empty")
+
   if (status) message("All intersections performed in latitude/longitude.")
 
   if(any(bbox > 180 | bbox < -180)) stop("invalid bbox entry")
@@ -155,27 +157,33 @@ subset_nhdplus <- function(comids = NULL, output_file = NULL, nhdplus_data = NUL
     nhdplus_data <- nhdplus_path()
   }
 
-  paths <- get_staged_data(nhdplus_data)
+  check_nhd_data(nhdplus_data)
 
   if(is.null(bbox)) {
     if(is.null(comids)) stop("must provide comids or bounding box")
 
     out_list <- c(get_flowline_subset(nhdplus_data, comids,
-                                      output_file, paths$fline_path,
-                                      status))
+                                      output_file,
+                                      status, out_prj))
 
     if(!flowline_only) {
+      tryCatch({
+        out_list <- c(out_list, get_catchment_subset(nhdplus_data, comids,
+                                                     output_file, simplified,
+                                                     status, out_prj))
 
-      out_list <- c(out_list, get_catchment_subset(nhdplus_data, comids,
-                                                   output_file, simplified,
-                                                   paths$catchment_path, status))
+        catch_layer <- get_catchment_layer_name(simplified, nhdplus_data)
 
-      catch_layer <- get_catchment_layer_name(simplified, nhdplus_data)
+        envelope <- sf::st_transform(sf::st_as_sfc(sf::st_bbox(out_list[[catch_layer]])),
+                                     4326)
 
-      envelope <- sf::st_transform(sf::st_as_sfc(sf::st_bbox(out_list[[catch_layer]])),
-                                   4326)
+        intersection_names <- c("NHDArea", "NHDWaterbody")
+      }, error = function(e) {
+        warning(paste("error getting catchment from nhdplus_data\n", e))
+      })
+
+      if(!exists("intersection_names")) intersection_names <- c()
 
-      intersection_names <- c("NHDArea", "NHDWaterbody")
     } else {
       intersection_names <- c()
     }
@@ -209,12 +217,16 @@ subset_nhdplus <- function(comids = NULL, output_file = NULL, nhdplus_data = NUL
       layer <- sf::st_transform(envelope, 4326) %>%
         get_nhdplus_bybox(layer = tolower(layer_name), streamorder = streamorder)
 
-      if(return_data) {
-        out_list[layer_name] <- list(layer)
-      }
+      if(nrow(layer) > 0) {
+        layer <- check_valid(layer, out_prj)
 
-      if(!is.null(output_file)) {
-        sf::write_sf(clean_bbox(layer), output_file, layer_name)
+        if(return_data) {
+          out_list[layer_name] <- list(layer)
+        }
+
+        if(!is.null(output_file)) {
+          sf::write_sf(clean_bbox(layer), output_file, layer_name)
+        }
       }
     }
 
@@ -233,7 +245,8 @@ subset_nhdplus <- function(comids = NULL, output_file = NULL, nhdplus_data = NUL
                                          data_path = nhdplus_data,
                                          envelope = envelope,
                                          output_file = output_file,
-                                         status), intersection_names))
+                                         status = status,
+                                         out_prj = out_prj), intersection_names))
   }
 
   if(return_data) return(out_list)
@@ -242,7 +255,7 @@ subset_nhdplus <- function(comids = NULL, output_file = NULL, nhdplus_data = NUL
 }
 
 intersection_write <- function(layer_name, data_path, envelope,
-                               output_file, status) {
+                               output_file, status, out_prj) {
   out_list <- list()
 
   if (status) message(paste("Reading", layer_name))
@@ -253,11 +266,20 @@ intersection_write <- function(layer_name, data_path, envelope,
   try(intersection_test <- suppressMessages(sf::st_intersects(
     sf::st_transform(layer, 4326), envelope)), silent = TRUE)
 
-  out <- dplyr::filter(layer, lengths(intersection_test) > 0)
+  found <- lengths(intersection_test)
+
+  if(length(found) > 0) {
+    out <- dplyr::filter(layer, found > 0)
+  } else {
+    out <- data.frame()
+  }
 
   if (nrow(out) > 0) {
+
+    out <- check_valid(out, out_prj)
+
     if (status) message(paste("Writing", layer_name))
-    if(is.null(output_file)) {
+    if (is.null(output_file)) {
       return(out)
     } else {
       sf::write_sf(clean_bbox(out), output_file, layer_name)
@@ -385,37 +407,21 @@ stage_national_data <- function(include = c("attribute",
 
 #' @title Try to find staged NHDPlus data
 #' @noRd
-get_staged_data <- function(nhdplus_data) {
+check_nhd_data <- function(nhdplus_data) {
 
-  if(nhdplus_data == "download") return(list(fline_path = NULL, catchment_path = NULL))
+  if(nhdplus_data == "download") return(invisible(TRUE))
 
-  staged_data <- try(get("national_data",
-                         envir = nhdplusTools_env),
-                     silent = TRUE)
-
-  if (is.list(staged_data)) {
-    if (all(c("flowline", "catchment") %in% names(staged_data)) &
-        file.exists(staged_data$flowline) &
-        file.exists(staged_data$catchment)) {
-      fline_path <- staged_data$flowline
-      catchment_path <- staged_data$catchment
-    } else {
-      fline_path <- nhdplus_data
-      catchment_path <- nhdplus_data
-    }
-  } else if(file.exists(nhdplus_data)) {
-    fline_path <- nhdplus_data
-    catchment_path <- nhdplus_data
+  if(file.exists(nhdplus_data)) {
+    return(invisible(TRUE))
   } else {
     stop("couldn't find nhdplus data")
   }
-  return(list(fline_path = fline_path, catchment_path = catchment_path))
 }
 
 #' @title Get subset of flowline data later.
 #' @noRd
 get_flowline_subset <- function(nhdplus_data, comids, output_file,
-                                fline_path, status) {
+                                status, out_prj) {
 
   layer_name <- get_flowline_layer_name(nhdplus_data)
 
@@ -428,21 +434,24 @@ get_flowline_subset <- function(nhdplus_data, comids, output_file,
     }
 
     fline <- get_nhdplus_byid(comids, tolower(layer_name))
+
   } else {
 
-    if (grepl("*.rds$", fline_path)) {
-      fline <- readRDS(fline_path)
-    } else {
-      if(!layer_name %in% st_layers(fline_path)$name) {
-        layer_name <- "NHDFlowline"
-      }
-      fline <- sf::read_sf(fline_path, layer_name)
-      fline <- align_nhdplus_names(fline)
+    if(!layer_name %in% st_layers(nhdplus_data)$name) {
+      layer_name <- "NHDFlowline"
     }
 
-    fline <- dplyr::filter(fline, .data$COMID %in% comids)
+
+
+    fline <- sf::read_sf(nhdplus_data, layer_name,
+                         query = get_query(nhdplus_data, layer_name,
+                                           "COMID", comids))
+    fline <- align_nhdplus_names(fline)
+
   }
 
+  fline <- check_valid(fline, out_prj)
+
   if (status) message(paste("Writing", layer_name))
 
   if(!is.null(output_file)) {
@@ -450,13 +459,28 @@ get_flowline_subset <- function(nhdplus_data, comids, output_file,
   }
   out <- list()
   out[layer_name] <- list(fline)
+
   return(out)
 }
 
+get_query <- function(nhdplus_data, layer_name, id, comids) {
+  layer_atts <- sf::read_sf(nhdplus_data, layer_name,
+                            query = paste0("SELECT * FROM ",
+                                           layer_name, " LIMIT 1"))
+
+  update_atts <- align_nhdplus_names(layer_atts)
+
+  id_att <- names(layer_atts)[which(names(update_atts) == id)]
+
+  query <- paste0("SELECT * FROM ", layer_name,
+                  " WHERE ", id_att, " IN (",
+                  paste(comids, collapse = ", "), ")")
+}
+
 #' @title Get subset of catchment data layer.
 #' @noRd
 get_catchment_subset <- function(nhdplus_data, comids, output_file,
-                                 simplified, catchment_path, status) {
+                                 simplified, status, out_prj) {
 
   layer_name <- get_catchment_layer_name(simplified, nhdplus_data)
 
@@ -468,17 +492,16 @@ get_catchment_subset <- function(nhdplus_data, comids, output_file,
 
   } else {
 
-    if (grepl("*.rds$", catchment_path)) {
-      catchment <- readRDS(catchment_path)
-    } else {
-      catchment <- sf::read_sf(catchment_path, layer_name)
-      catchment <- align_nhdplus_names(catchment)
-    }
+    catchment <- sf::read_sf(nhdplus_data, layer_name,
+                             query = get_query(nhdplus_data, layer_name,
+                                               "FEATUREID", comids))
 
-    catchment <- dplyr::filter(catchment, .data$FEATUREID %in% comids)
+    catchment <- align_nhdplus_names(catchment)
 
   }
 
+  catchment <- check_valid(catchment, out_prj)
+
   if (status) message(paste("Writing", layer_name))
 
   if(!is.null(output_file)) {
@@ -493,6 +516,59 @@ clean_bbox <- function(x) {
   if("bbox" %in% names(x) && class(x$bbox[1]) == "list") {
     x$bbox <- sapply(x$bbox, paste, collapse = ",")
   }
+
+  return(x)
+}
+
+check_valid <- function(x, out_prj) {
+
+  x <- sf::st_zm(x)
+
+  if (!all(sf::st_is_valid(x))) {
+
+    message("Found invalid geometry, attempting to fix.")
+
+    try(x <- sf::st_make_valid(x))
+  }
+
+  if (any(grepl("POLYGON", class(sf::st_geometry(x))))) {
+    suppressMessages(suppressWarnings(x <- sf::st_buffer(x, 0)))
+  }
+
+  if (sf::st_crs(x) != sf::st_crs(out_prj)) {
+    x <- sf::st_transform(x, out_prj)
+  }
+
+  types <- as.character(sf::st_geometry_type(x, by_geometry = FALSE))
+
+  if(grepl("^GEOME", types)) {
+    unq <- unique(as.character(
+      sf::st_geometry_type(x, by_geometry = TRUE)))
+
+    cast_to <- unq[which.max(tabulate(match(types, unq)))]
+
+    if(any(grepl("^MULTI", unq)) & !grepl("^MULTI", cast_to)) {
+      cast_to <- paste0("MULTI", cast_to)
+    }
+
+    tryCatch(x <- sf::st_cast(x, cast_to),
+             error = function(e) {
+               warning(paste0("\n\n Failed to unify output geometry type. \n\n",
+                             e,
+                             "\n Dropping non-", cast_to, " geometries. \n"))
+             })
+
+    r <- nrow(x)
+
+    x <- x[sf::st_geometry_type(x, by_geometry = TRUE) == cast_to, ]
+
+    if(r != nrow(x)) {
+      x <- sf::st_cast(x, cast_to)
+    }
+  }
+
+  suppressWarnings(x <- sf::st_simplify(x, dTolerance = 0))
+
   return(x)
 }
 
