@@ -1,3 +1,8 @@
+library("sf")
+library("dplyr")
+
+unlink(file.path(tempdir(check = TRUE), "*"), recursive = TRUE)
+
 get_test_file <- function(temp_dir) {
   check_locations <- c("../../docs/data/03_sub.zip", "docs/data/03_sub.zip")
   check_location <- check_locations[file.exists(check_locations)]
